// INCLUDES ///////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN  

// has the GUID library been included?
#ifndef INITGUID
#define INITGUID
#endif

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <direct.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <io.h>
#include <fcntl.h>
#include <objbase.h>
#include <fstream.h>

// DirectX Includes
#include <ddraw.h>		//DirectDraw  
#include <dinput.h>		//DirectInput
#include <dsound.h>		//DirectSound
#include <dmksctrl.h>	//DirectMusic
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>

#include "DXLib.h"

// Macros
// initializes a direct draw struct
#define DDraw_InitStruct(ddstruct) { memset(&ddstruct,0,sizeof(ddstruct)); ddstruct.dwSize=sizeof(ddstruct); }
#define MULTI_TO_WIDE( x,y )  MultiByteToWideChar( CP_ACP,MB_PRECOMPOSED, y,-1,x,_MAX_PATH);

// DEFINES ////////////////////////////////////////////////

// TYPES //////////////////////////////////////////////////

// PROTOTYPES /////////////////////////////////////////////

// EXTERNALS /////////////////////////////////////////////

extern HWND main_window_handle; // save the window handle
extern HINSTANCE main_instance; // save the instance

// GLOBALS ////////////////////////////////////////////////
// sin and cos table
float sint[360];
float cost[360];

// notice that interface 4.0 is used on a number of interfaces
LPDIRECTDRAW7        lpdd         = NULL;  // dd object
LPDIRECTDRAWSURFACE7 lpddsprimary = NULL;  // dd primary surface
LPDIRECTDRAWSURFACE7 lpddsback    = NULL;  // dd back surface
LPDIRECTDRAWCLIPPER  lpddclipper  = NULL;  // dd clipper
DDSURFACEDESC2       ddsd;                 // a direct draw surface description struct
USHORT               *primary_buffer = NULL; // primary video buffer
USHORT               *back_buffer    = NULL; // secondary back buffer
int                  primary_lpitch  = 0;    // memory line pitch for primary buffer
int                  back_lpitch     = 0;    // memory line pitch for back buffer

// DirectInput
LPDIRECTINPUT lpdi=NULL;	// di object
LPDIRECTINPUTDEVICE lpdikey=NULL; // DirectInput keyboard device
LPDIRECTINPUTDEVICE lpdimouse=NULL; // DirectInput mouse device
UCHAR keystate[256];
DIMOUSESTATE mousestate;

// DirectSound
LPDIRECTSOUND lpds = NULL;	// the directsound object
DSBUFFERDESC dsbd;	// ds buffer desc
WAVEFORMATEX pcmfx;	// holds format desc

// DirectMusic
IDirectMusicPerformance	*dmp = NULL;	// DirectMusic Performance
IDirectMusicLoader *dml = NULL;			// DirectMusic Loader

// FUNCTIONS //////////////////////////////////////////////
// sin and cos tables
void TableSetup (void)
{
	for (int d=0; d<360; d++)
	{
		double rd = d * 3.14159/180;

		sint[d] = sin(rd);
		cost[d] = cos(rd);
	}
}

// Particle system
CParticle::CParticle (float x, float y, float dx, float dy, USHORT startcolor, USHORT endcolor, int lifespan, int size, bool fade_blink, bool alpha, bool vel_random, bool wrap, bool bounce, bool constant)
{
	this->x = x;
	this->y = y;
	this->color = startcolor;
	this->startcolor = startcolor;
	this->constant = constant;
	this->endcolor = endcolor;
	this->size = size;
	this->lifespan = lifespan;
	this->wrap = wrap;
	this->vel_random = vel_random;
	this->alpha = alpha;
	this->dx = dx;
	this->dy = dy;
	this->fade_blink = fade_blink;
	this->bounce = bounce;
	if (fade_blink)
	{	// fade
		if ((GETR(endcolor)-GETR(startcolor))<0)
		{
			rmax = (-lifespan)/(GETR(endcolor)-GETR(startcolor));
		}
		else if ((GETR(endcolor)-GETR(startcolor))>0)
		{
			rmax = lifespan/(GETR(endcolor)-GETR(startcolor));
		}
		else
			rmax = -1;

		rcounter = rmax;

		if ((GETG(endcolor)-GETG(startcolor))<0)
		{
			gmax = (-lifespan)/(GETG(endcolor)-GETG(startcolor));
		}
		else if ((GETG(endcolor)-GETG(startcolor))>0)
		{
			gmax = lifespan/(GETG(endcolor)-GETG(startcolor));
		}
		else
			gmax = -1;

		gcounter = gmax;

		if ((GETB(endcolor)-GETB(startcolor))<0)
		{
			bmax = (-lifespan)/(GETB(endcolor)-GETB(startcolor));
		}
		else if ((GETB(endcolor)-GETB(startcolor))>0)
		{
			bmax = lifespan/(GETB(endcolor)-GETB(startcolor));
		}
		else
			bmax = -1;

		bcounter = bmax;
	}
}

void CParticle::setupFade (void)
{
		if ((GETR(endcolor)-GETR(startcolor))<0)
		{
			rmax = (-lifespan)/(GETR(endcolor)-GETR(startcolor));
		}
		else if ((GETR(endcolor)-GETR(startcolor))>0)
		{
			rmax = lifespan/(GETR(endcolor)-GETR(startcolor));
		}
		else
			rmax = -1;

		rcounter = rmax;

		if ((GETG(endcolor)-GETG(startcolor))<0)
		{
			gmax = (-lifespan)/(GETG(endcolor)-GETG(startcolor));
		}
		else if ((GETG(endcolor)-GETG(startcolor))>0)
		{
			gmax = lifespan/(GETG(endcolor)-GETG(startcolor));
		}
		else
			gmax = -1;

		gcounter = gmax;

		if ((GETB(endcolor)-GETB(startcolor))<0)
		{
			bmax = (-lifespan)/(GETB(endcolor)-GETB(startcolor));
		}
		else if ((GETB(endcolor)-GETB(startcolor))>0)
		{
			bmax = lifespan/(GETB(endcolor)-GETB(startcolor));
		}
		else
			bmax = -1;

		bcounter = bmax;
}

CParticle::CParticle (float x, float y, USHORT startcolor, USHORT endcolor, int lifespan, int size)
{
	this->x = x;
	this->y = y;
	this->color = startcolor;
	this->size = size;
	this->startcolor = startcolor;
	this->endcolor = endcolor;
	this->lifespan = lifespan;
	wrap = false;
	alpha = false;
	vel_random = true;
	fade_blink = false;
	constant = true;
	bounce = false;
}

void CParticle::draw ( void )
{
	if (alive)
	{
	USHORT drawcolor = color;
	int cur_y, cur_x;
	for (int add_y = 0; add_y<size; add_y++)
	{
		cur_y = y+add_y;
		for (int add_x = 0; add_x<size; add_x++)
		{
			cur_x = x+add_x;
			USHORT tempclr = back_buffer[cur_x + (back_lpitch * cur_y)];

			if (alpha)	// particle is 50% transparent
			{
				int cr,cb,cg,or,ob,og;
				cr = GETR(color);
				cb = GETB(color);
				cg = GETG(color);
				or = GETR(tempclr);
				ob = GETB(tempclr);
				og = GETB(tempclr);
				drawcolor = RGB216B((cr+cb) >> 1,(cb+ob) >> 1,(cg+og) >> 1);
			}
			back_buffer[cur_x + (back_lpitch * cur_y)] = drawcolor;
		}
	}
	}
}

void CParticle::move ( void )
{
	if (vel_random)
	{
		x += 3-(rand()%6);
		y += 3-(rand()%6);

		if (x < 0)
		{
			if (wrap)
			{
				x = xRes - size;
			}
			else if (bounce)
			{
				dx = -dx;
				x += dx/2;
			}
			else	// no nothing
			{
				alive = false;	// off screen
			}
		}
		if (y < 0)
		{
			if (wrap)
			{
				y = yRes - size;
			}
			else if (bounce)
			{
				dy = -dy;
				y += dy/2;
			}
			else	// no nothing
			{
				alive = false;	// off screen
			}
		}
		if (x > (xRes-size))
		{
			if (wrap)
			{
				x = 0;
			}
			else if (bounce)
			{
				dx = -dx;
				x += dx/2;
			}
			else	// no nothing
			{
				alive = false;	// off screen
			}
		}
		if (y > (yRes-size))
		{
			if (wrap)
			{
				y = 0;
			}
			else if (bounce)
			{
				dy = -dy;
				y += dy/2;
			}
			else	// no nothing
			{
				alive = false;	// off screen
			}
		}
	}
	else	// velocity on
	{
		x += dx;
		y += dy;
	
		if (x < 0)
		{
			if (wrap)
			{
				x = xRes - size;
			}
			else if (bounce)
			{
				dx = -dx;
				x += dx/2;
			}
			else	// no nothing
			{
				alive = false;	// off screen
			}
		}
		if (y < 0)
		{
			if (wrap)
			{
				y = yRes - size;
			}
			else if (bounce)
			{
				dy = -dy;
				y += dy/2;
			}
			else	// no nothing
			{
				alive = false;	// off screen
			}
		}
		if (x > (xRes-size))
		{
			if (wrap)
			{
				x = 0;
			}
			else if (bounce)
			{
				dx = -dx;
				x += dx/2;
			}
			else	// no nothing
			{
				alive = false;	// off screen
			}
		}
		if (y > (yRes-size))
		{
			if (wrap)
			{
				y = 0;
			}
			else if (bounce)
			{
				dy = -dy;
				y += dy/2;
			}
			else	// no nothing
			{
				alive = false;	// off screen
			}
		}
	}

	if (lifespan>-1)
	{
		lifespan--;
		if (lifespan <= 0)
			alive = false;
	}

	if (!constant)
	{
		if ((fade_blink)&&(alive))
		{	// fade
			if (rmax > -1)
				rcounter--;

			if (gmax > -1)
				gcounter--;

			if (bmax > -1)
				bcounter--;

			if ((rcounter==0) && (rmax > -1))
			{
				rcounter = rmax;
				if ((GETR(endcolor)-GETR(startcolor)) < 0)
				{
					color = RGB216B(GETR(color)-1,GETG(color),GETB(color));
				}
				else if ((GETR(endcolor)-GETR(startcolor)) > 0)
				{
					color = RGB216B(GETR(color)+1,GETG(color),GETB(color));
				}
				if (GETR(color)==GETR(endcolor))
					rmax = -1;	// red fading off
			}

			if ((gcounter==0) && (gmax > -1))
			{
				gcounter = gmax;
				if ((GETG(endcolor)-GETG(startcolor)) < 0)
				{
					color = RGB216B(GETR(color),GETG(color)-1,GETB(color));
				}
				else if ((GETG(endcolor)-GETG(startcolor)) > 0)
				{
					color = RGB216B(GETR(color),GETG(color)+1,GETB(color));
				}
				if (GETG(color) == GETG(endcolor))
					gmax = -1;
			}

			if ((bcounter==0) && (bmax > -1))
			{
				bcounter = bmax;
				if ((GETB(endcolor)-GETB(startcolor)) < 0)
				{
					color = RGB216B(GETR(color),GETG(color),GETB(color)-1);
				}
				else if ((GETB(endcolor)-GETB(startcolor)) > 0)
				{
					color = RGB216B(GETR(color),GETG(color),GETB(color)+1);
				}
				if (GETB(color) == GETB(endcolor))
					bmax = -1;
			}
		}
		//else if (!fade_blink)
		//{	// blink
		//	color = RGB216B(rand()%31,rand()%62,rand()%31);
		//}
	}
}

// DirectMusic
int CMusic::LoadMIDI (char *filename)
{
DMUS_OBJECTDESC ObjDesc; 
HRESULT hr;
IDirectMusicSegment* pSegment = NULL;

// get current working directory
char szDir[_MAX_PATH];
WCHAR wszDir[_MAX_PATH]; 

if(_getcwd( szDir, _MAX_PATH ) == NULL)
  {
  return 0;
  } // end if

MULTI_TO_WIDE(wszDir, szDir);

// tell the loader were to look for files
hr = dml->SetSearchDirectory(GUID_DirectMusicAllTypes,wszDir, FALSE);

if (FAILED(hr)) 
   {
   return 0;
   } // end if

// convert filename to wide string
WCHAR wfilename[_MAX_PATH]; 
MULTI_TO_WIDE(wfilename, filename);
 
// setup object description
DDraw_InitStruct(ObjDesc);
ObjDesc.guidClass = CLSID_DirectMusicSegment;
wcscpy(ObjDesc.wszFileName, wfilename );
ObjDesc.dwValidData = DMUS_OBJ_CLASS | DMUS_OBJ_FILENAME;
 
// load the object and query it for the IDirectMusicSegment interface
// This is done in a single call to IDirectMusicLoader::GetObject
// note that loading the object also initializes the tracks and does 
// everything else necessary to get the MIDI data ready for playback.

hr = dml->GetObject(&ObjDesc,IID_IDirectMusicSegment, (void**) &pSegment);

if (FAILED(hr))
   return 0;
 
// ensure that the segment plays as a standard MIDI file
// you now need to set a parameter on the band track
// Use the IDirectMusicSegment::SetParam method and let 
// DirectMusic find the trackby passing -1 (or 0xFFFFFFFF) in the dwGroupBits method parameter.

hr = pSegment->SetParam(GUID_StandardMIDIFile,-1, 0, 0, (void*)dmp);

if (FAILED(hr))
   return 0;
  
// This step is necessary because DirectMusic handles program changes and 
// bank selects differently for standard MIDI files than it does for MIDI 
// content authored specifically for DirectMusic. 
// The GUID_StandardMIDIFile parameter must be set before the instruments are downloaded. 

// The next step is to download the instruments. 
// This is necessary even for playing a simple MIDI file 
// because the default software synthesizer needs the DLS data 
// for the General MIDI instrument set
// If you skip this step, the MIDI file will play silently.
// Again, you call SetParam on the segment, this time specifying the GUID_Download parameter:

hr = pSegment->SetParam(GUID_Download, -1, 0, 0, (void*)dmp);

if (FAILED(hr))
   return 0;

// at this point we have MIDI loaded and a valid object

dms  = pSegment;
dmss = NULL;
state       = 1;
 
// return id
return 1;
}

int CMusic::Play ( void )
{
	if (FAILED(dmp->PlaySegment (dms, 0, 0, &dmss)))
	{
		MessageBox (main_window_handle,"Error","Unable to play music segment",0);
		return 1;
	}
	return 1;
}

int CMusic::Stop ( void )
{
	if (FAILED(dmp->Stop (dms,NULL,0,0)))
	{
		MessageBox (main_window_handle,"Error","Unable to stop segment",0);
		return 0;
	}
	return 1;
}

bool CMusic::isPlaying ( void )
{
	if (dmp->IsPlaying (dms,NULL) == S_OK)
	{
		return true;
	}
	else
	{
		return false;
	}
}

int CMusic::Release ( void )
{
	dms->SetParam (GUID_Unload, -1, 0, 0, (void*)dmp);
	dms->Release ();
	dms = NULL;
	return 1;
}

int DMusic_Setup ( void )
{
	if (FAILED(CoInitialize (NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to setup COM for DM",0);
		return 0;
	}

	if (FAILED(CoCreateInstance (CLSID_DirectMusicPerformance, NULL, CLSCTX_INPROC, IID_IDirectMusicPerformance, (void**)&dmp)))
	{
		MessageBox (main_window_handle,"Error","Unable to create DMPerformance COM object",0);
		return 0;
	}

	if (lpds)
	{
	if (FAILED(dmp->Init (NULL,lpds,main_window_handle)))
	{
		MessageBox (main_window_handle,"Error","Unable to Initialize DM",0);
		return 0;
	}
	}
	else
	{
	if (FAILED(dmp->Init (NULL,NULL,main_window_handle)))
	{
		MessageBox (main_window_handle,"Error","Unable to Initialize DM",0);
		return 0;
	}
	}

	if (FAILED(dmp->AddPort (NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to add port to performance",0);
		return 0;
	}

	if (FAILED(CoCreateInstance (CLSID_DirectMusicLoader, NULL, CLSCTX_INPROC,IID_IDirectMusicLoader, (void**)&dml)))
	{
		MessageBox (main_window_handle,"Error","Unable to create DM Loader",0);
		return 0;
	}

	// Finito
	return 1;
}

int DMusic_Release ( void )
{
	if (dmp)
	{
		dmp->Stop (NULL,NULL,0,0);
		dmp->CloseDown ();
		dmp->Release ();
	}
	if (dml)
		dml->Release ();

	CoUninitialize ();
	return 1;
}

// DirectSound
int CSound::Setup (int size, int flags)
{
	this->size = size;
	this->rate = rate;

	DDraw_InitStruct (dsbd);
	memset (&pcmfx,0,sizeof(pcmfx));
	pcmfx.wFormatTag = WAVE_FORMAT_PCM;
	pcmfx.nChannels = 1;			// mono
	pcmfx.nSamplesPerSec = 11025;	// 11kHz/22kHz/etc
	pcmfx.nBlockAlign = 1;
	pcmfx.nAvgBytesPerSec = pcmfx.nSamplesPerSec * pcmfx.nBlockAlign;
	pcmfx.wBitsPerSample = 8;		// 8bit
	pcmfx.cbSize = 0;

	dsbd.dwFlags = flags;
	dsbd.dwBufferBytes = size;	// # of sec  *  rate
	dsbd.lpwfxFormat = &pcmfx;

	if (FAILED(lpds->CreateSoundBuffer (&dsbd,&buffer,NULL)))
	{
		MessageBox (main_window_handle,"Error","Cannot create sound buffer",0);
	}

	state = 1;	// ready
	return 1;
}

int CSound::Pause ( void )
{
	if (state > 1)
		buffer->Stop ();
	state = 1;
	return 1;
}

int CSound::Release ()
{
	state = 0;
	buffer->Release ();
	buffer = NULL;
	return 1;
}

int CSound::Play (bool loop)
{
	int flags = 0;
	if (loop)
		flags = DSBPLAY_LOOPING;

	if (FAILED(buffer->Play (0,0,flags)))
	{
		MessageBox (main_window_handle,"Error","Unable to Play sound!",0);
		return 0;
	}

	state = 2;	// Playing
	if (loop)
		state = 3;	// Looping
	return 1;
}

int CSound::Stop ( void )
{
	if (state > 1)
	{
		buffer->Stop ();
		buffer->SetCurrentPosition (0);
		state = 1;	// ready to rumble
	}
	return 1;
}

int CSound::SetVolume ( int volume )
{
	if (FAILED(buffer->SetVolume ((DWORD)(-30*(100 - volume)))))
	{
		MessageBox (main_window_handle,"Error","Unable to set volume",0);
		return 0;
	}
	return 1;
}

int CSound::SetFrequency ( int freq )
{
	if (FAILED(buffer->SetFrequency (freq)))
	{
		MessageBox (main_window_handle,"Error","Cannot set frequency",0);
		return 0;
	}
	return 1;
}

int CSound::SetPan ( int pan )
{
	if (FAILED(buffer->SetPan (pan)))
	{
		MessageBox (main_window_handle,"Error","Unable to pan sound",0);
		return 0;
	}
	return 1;
}

int CSound::LoadSound ( char *name )
{
// this function loads a .wav file, sets up the directsound 
// buffer and loads the data into memory, the function returns 
// the id number of the sound


HMMIO 			hwav;    // handle to wave file
MMCKINFO		parent,  // parent chunk
                child;   // child chunk
WAVEFORMATEX    wfmtx;   // wave format structure

UCHAR *snd_buffer,       // temporary sound buffer to hold voc data
      *audio_ptr_1=NULL, // data ptr to first write buffer 
	  *audio_ptr_2=NULL; // data ptr to second write buffer

DWORD audio_length_1=0,  // length of first write buffer
	  audio_length_2=0;  // length of second write buffer
			
// set up chunk info structure
parent.ckid 	    = (FOURCC)0;
parent.cksize 	    = 0;
parent.fccType	    = (FOURCC)0;
parent.dwDataOffset = 0;
parent.dwFlags		= 0;

// copy data
child = parent;

// open the WAV file
if ((hwav = mmioOpen(name, NULL, MMIO_READ | MMIO_ALLOCBUF))==NULL)
    return 0;

// descend into the RIFF 
parent.fccType = mmioFOURCC('W', 'A', 'V', 'E');

if (mmioDescend(hwav, &parent, NULL, MMIO_FINDRIFF))
    {
    // close the file
    mmioClose(hwav, 0);

    // return error, no wave section
    return 0; 	
    } // end if

// descend to the WAVEfmt 
child.ckid = mmioFOURCC('f', 'm', 't', ' ');

if (mmioDescend(hwav, &child, &parent, 0))
    {
    // close the file
    mmioClose(hwav, 0);

    // return error, no format section
    return 0; 	
    } // end if

// now read the wave format information from file
if (mmioRead(hwav, (char *)&wfmtx, sizeof(wfmtx)) != sizeof(wfmtx))
    {
    // close file
    mmioClose(hwav, 0);

    // return error, no wave format data
    return 0;
    } // end if

// make sure that the data format is PCM
if (wfmtx.wFormatTag != WAVE_FORMAT_PCM)
    {
    // close the file
    mmioClose(hwav, 0);

    // return error, not the right data format
    return 0; 
    } // end if

// now ascend up one level, so we can access data chunk
if (mmioAscend(hwav, &child, 0))
   {
   // close file
   mmioClose(hwav, 0);

   // return error, couldn't ascend
   return 0; 	
   } // end if

// descend to the data chunk 
child.ckid = mmioFOURCC('d', 'a', 't', 'a');

if (mmioDescend(hwav, &child, &parent, MMIO_FINDCHUNK))
    {
    // close file
    mmioClose(hwav, 0);

    // return error, no data
    return 0; 	
    } // end if

// finally!!!! now all we have to do is read the data in and
// set up the directsound buffer

// allocate the memory to load sound data
snd_buffer = (UCHAR *)malloc(child.cksize);

// read the wave data 
mmioRead(hwav, (char *)snd_buffer, child.cksize);

// close the file
mmioClose(hwav, 0);

// set rate and size in data structure
rate  = wfmtx.nSamplesPerSec;
size  = child.cksize;
state = 1;

// copy data into sound buffer
if (FAILED(buffer->Lock(0, child.cksize, (void **) &audio_ptr_1, &audio_length_1, (void **)&audio_ptr_2, &audio_length_2, DSBLOCK_FROMWRITECURSOR)))
	 return(0);

// copy first section of circular buffer
memcpy(audio_ptr_1, snd_buffer, audio_length_1);

// copy last section of circular buffer
memcpy(audio_ptr_2, (snd_buffer+audio_length_1),audio_length_2);

// unlock the buffer
if (FAILED(buffer->Unlock(audio_ptr_1, 
									    audio_length_1, 
									    audio_ptr_2, 
									    audio_length_2)))
 							     return(0);

// release the temp buffer
free(snd_buffer);

// return id
return 1;
}

int DSound_Setup ( void )
{
	if (FAILED(DirectSoundCreate (NULL,&lpds,NULL)))
	{
		return 0;
	}
	if (FAILED(lpds->SetCooperativeLevel (main_window_handle,DSSCL_NORMAL)))
	{
		return 0;
	}
	// Ready to rock and roll
	return 1;
}

int DSound_Release ( void )
{
	if (lpds)
		lpds->Release ();
	return 1;
}

// DIRECT DRAW
// Graphic Primitives
int DDraw_fillSecondary (int x, int y, USHORT c, USHORT background)
{
	if (back_buffer[x + (back_lpitch*y)] == background)
	{
		back_buffer[x + (back_lpitch*y)] = c;
		DDraw_fillSecondary (x, y-1, c, background);
		DDraw_fillSecondary (x, y+1, c, background);
		DDraw_fillSecondary (x-1, y, c, background);
		DDraw_fillSecondary (x+1, y, c, background);
	}
		return 1;
}

int DDraw_drawLine(int x0, int y0, int x1, int y1, int color, USHORT *vb_start, int lpitch_2)
{
// this function draws a line from xo,yo to x1,y1 using differential error
// terms (based on Bresenahams work)

int dx,             // difference in x's
    dy,             // difference in y's
    dx2,            // dx,dy * 2
    dy2, 
    x_inc,          // amount in pixel space to move during drawing
    y_inc,          // amount in pixel space to move during drawing
    error,          // the discriminant i.e. error i.e. decision variable
    index;          // used for looping

// pre-compute first pixel address in video buffer based on 16bit data
USHORT *vb_start2 = (USHORT *)vb_start + x0 + y0*lpitch_2;

// compute horizontal and vertical deltas
dx = x1-x0;
dy = y1-y0;

// test which direction the line is going in i.e. slope angle
if (dx>=0)
   {
   x_inc = 1;

   } // end if line is moving right
else
   {
   x_inc = -1;
   dx    = -dx;  // need absolute value

   } // end else moving left

// test y component of slope

if (dy>=0)
   {
   y_inc = lpitch_2;
   } // end if line is moving down
else
   {
   y_inc = -lpitch_2;
   dy    = -dy;  // need absolute value

   } // end else moving up

// compute (dx,dy) * 2
dx2 = dx << 1;
dy2 = dy << 1;

// now based on which delta is greater we can draw the line
if (dx > dy)
   {
   // initialize error term
   error = dy2 - dx; 

   // draw the line
   for (index=0; index <= dx; index++)
       {
       // set the pixel
       *vb_start2 = (USHORT)color;

       // test if error has overflowed
       if (error >= 0) 
          {
          error-=dx2;

          // move to next line
          vb_start2+=y_inc;

	   } // end if error overflowed

       // adjust the error term
       error+=dy2;

       // move to the next pixel
       vb_start2+=x_inc;

       } // end for

   } // end if |slope| <= 1
else
   {
   // initialize error term
   error = dx2 - dy; 

   // draw the line
   for (index=0; index <= dy; index++)
       {
       // set the pixel
       *vb_start2 = (USHORT)color;

       // test if error overflowed
       if (error >= 0)
          {
          error-=dy2;

          // move to next line
          vb_start2+=x_inc;

          } // end if error overflowed

       // adjust the error term
       error+=dx2;

       // move to the next pixel
       vb_start2+=y_inc;

       } // end for

   } // end else |slope| > 1

// return success
return(1);

} // end DDraw_drawLine by Andre LaMothe

int DDraw_drawString (char *text, int x, int y, COLORREF color, LPDIRECTDRAWSURFACE7 surface)
{
	HDC gdc;

	if (FAILED(surface->GetDC(&gdc)))
	{
		MessageBox(main_window_handle,"Error","Unable to get DC!",0);
		return 0;
	}
	SetTextColor(gdc,color);
	SetBkMode(gdc,TRANSPARENT);
	TextOut(gdc,x,y,text,strlen(text));
	surface->ReleaseDC(gdc);
	return 1;
}

int DDraw_fillRect (int x1, int y1, int x2, int y2, USHORT c, LPDIRECTDRAWSURFACE7 surface)
{
	DDBLTFX ddbltfx;
	RECT dest;

	DDraw_InitStruct (ddbltfx);
	ddbltfx.dwFillColor = c;
	dest.left = x1;
	dest.top = y1;
	dest.right = x2;
	dest.bottom = y2;

	HRESULT hr;
	hr = surface->Blt(&dest,NULL,NULL,DDBLT_COLORFILL|DDBLT_WAIT,&ddbltfx);

	if (hr != DD_OK)
	{
		if (hr == DDERR_SURFACELOST)
		{
			return 2;	// Surface Lost
		}
		return 0;
	} // end if
	return 1;
}


// Surface functions
int DDraw_Transition (int Transition)
{
	DWORD timer = GetTickCount ();
	LPDIRECTDRAWSURFACE7 copy = NULL;
	copy = DDraw_CreateSurface (800,600,DDSCAPS_SYSTEMMEMORY);
	DDraw_Blit (0,0,800,600,lpddsprimary,copy,false);

	switch (Transition)
	{
	case TRANSITION_FADETOBLACK:
		{
			bool done = false;

			while (!done)
			{
			DDraw_fillRect (0,0,800,600,0,lpddsback);

			int pitch = 0;
			USHORT *copy_buffer = DDraw_LockSurface (copy,&pitch);
			
			done = true;
			for (int y=0; y<yRes; y++)
			{
				for (int x=0; x<xRes; x++)
				{
					USHORT c = copy_buffer[x+(pitch*y)];

					if (GETR(c)>0)
					{
						c = RGB216B(GETR(c)- ((GETR(c)>>3) ? (GETR(c)>>3) : 1),GETG(c),GETB(c));
						done = false;
					}
					if (GETG(c)>0)
					{
						c = RGB216B(GETR(c),GETG(c)- ((GETG(c)>>3) ? (GETG(c)>>3) : 1), GETB(c));
						done = false;
					}
					if (GETB(c)>0)
					{
						c = RGB216B(GETR(c),GETG(c),GETB(c)- ((GETB(c)>>3) ? (GETB(c)>>3) : 1));
						done = false;
					}
					copy_buffer[x+(y*pitch)] = c;
				}
			}
			DDraw_UnlockSurface (copy);

			DDraw_Blit (0,0,800,600,copy,lpddsback,false);
			DDraw_Flip ();
			while ((GetTickCount()-timer)<(1000/30));
			timer = GetTickCount();
			DInput_KeyboardRead ();
			if ((keystate[DIK_RETURN] & 0x80)||(keystate[DIK_ESCAPE] & 0x80))
				done = true;
			}
		} break;
	case TRANSITION_FADETOWHITE:
		{
			bool done = false;

			while (!done)
			{
			DDraw_fillRect (0,0,800,600,0,lpddsback);

			int pitch = 0;
			USHORT *copy_buffer = DDraw_LockSurface (copy,&pitch);
			
			done = true;
			for (int y=0; y<yRes; y++)
			{
				for (int x=0; x<xRes; x++)
				{
					USHORT c = copy_buffer[x+(pitch*y)];

					if (GETR(c)<31)
					{
						c = RGB216B(GETR(c)+ ((((GETR(c)>>3)>0)&&(((GETR(c)>>3)+GETR(c))<32)) ? GETR(c)>>3 : 1),GETG(c),GETB(c));
						done = false;
					}
					if (GETG(c)<63)
					{
						c = RGB216B(GETR(c),GETG(c)+ ((((GETG(c)>>3)>0)&&(((GETG(c)>>3)+GETG(c))<64)) ? GETG(c)>>3 : 1), GETB(c));
						done = false;
					}
					if (GETB(c)<31)
					{
						c = RGB216B(GETR(c),GETG(c),GETB(c)+ ((((GETB(c)>>3)>0)&&(((GETB(c)>>3)+GETB(c))<32)) ? GETB(c)>>3 : 1));
						done = false;
					}
					copy_buffer [x + (y*pitch)] = c;
				}
			}
			DDraw_UnlockSurface (copy);

			DDraw_Blit (0,0,800,600,copy,lpddsback,false);
			DDraw_Flip ();
			while ((GetTickCount()-timer)<(1000/30));
			timer = GetTickCount();
			DInput_KeyboardRead ();
			if ((keystate[DIK_RETURN] & 0x80)||(keystate[DIK_ESCAPE] & 0x80))
				done = true;
			}
		} break;
	case TRANSITION_BLUR:
		{
			int blurriness = 6 + rand()%3;

			for (int b=0; b<blurriness; b++)
			{
				DDraw_fillRect (0,0,800,600,0,lpddsback);

				int pitch = 0;
				USHORT *copy_buffer = DDraw_LockSurface (copy,&pitch);

				for (int y=1; y<(yRes-1); y++)
				{
					for (int x=1; x<(xRes-1); x++)
					{
						int cr,g,b;
						USHORT c,r,l,u,d;
						c = copy_buffer[x+(pitch*y)];
						r = copy_buffer[(x+1)+(pitch*y)];
						l = copy_buffer[(x-1)+(pitch*y)];
						u = copy_buffer[x+(pitch*(y+1))];
						d = copy_buffer[x+(pitch*(y-1))];
						cr = GETR(c)+GETR(u)+GETR(d)+GETR(l)+GETR(r);
						cr = cr/5;
						g = GETG(c)+GETG(u)+GETG(d)+GETG(l)+GETG(r);
						g = g/5;
						b = GETB(c)+GETB(u)+GETB(d)+GETB(l)+GETB(r);
						b = b/5;
						copy_buffer[x+(pitch*y)]=RGB216B(cr,g,b);
					}
				}
				DDraw_UnlockSurface (copy);
				DDraw_Blit (0,0,800,600,copy,lpddsback,false);
				DDraw_Flip ();
				while ((GetTickCount() - timer)<(1000/30));
				timer = GetTickCount();
				DInput_KeyboardRead ();
				if ((keystate[DIK_RETURN] & 0x80)||(keystate[DIK_ESCAPE] & 0x80))
					b = blurriness;
			}
		} break;
	case TRANSITION_CRUNCH:
		{
			for (int dy = 0; dy<297; dy+=4)
			{
				DDraw_fillRect (0,0,800,600,0,lpddsback);
				DDraw_Blit (0,dy,800,(600-dy)-dy,copy,lpddsback,false);
				DDraw_Flip ();
				if ((GetTickCount() - timer)<(1000/30));
				timer = GetTickCount ();
			}
			for (dy = 0; dy<393; dy+=8)
			{
				DDraw_fillRect (0,0,800,600,0,lpddsback);
				DDraw_Blit (dy,299,(800-dy)-dy,3,copy,lpddsback,false);
				DDraw_Flip ();
				if ((GetTickCount () - timer)<(1000/30));
				timer = GetTickCount();
			}
		} break;
	case TRANSITION_DISSOLVE:
		{
			for (int count=0; count<(90+rand()%90); count++)
			{
				DDraw_fillRect (0,0,800,600,0,lpddsback);

				int pitch = 0;
				USHORT *copy_buffer = DDraw_LockSurface (copy,&pitch);
				
				for (int p=0; p<(1500+rand()%1000); p++)
				{
					copy_buffer [(rand()%800) + (pitch * (rand()%600))] = 0;
				}
				
				DDraw_UnlockSurface (copy);
				DDraw_Blit (0,0,800,600,copy,lpddsback,false);
				DDraw_Flip ();
				while ((GetTickCount() - timer)<(1000/30));
				timer = GetTickCount();
			}
		} break;
	case TRANSITION_TELEPORT:
		{
			int pitch = 0;
			USHORT *copy_buffer = DDraw_LockSurface (copy,&pitch);
			DDraw_fillRect (0,0,800,600,0,lpddsprimary);
			for (int count=0; count<(90+rand()%90); count++)
			{
				DDraw_fillRect (0,0,800,600,0,lpddsback);
				DDraw_Blit (0,0,800,600,lpddsprimary,lpddsback,false);
				DDraw_SecondaryLock ();

				for (int p=0; p<(20000); p++)
				{
					int x,y;
					x = rand()%800;
					y = rand()%600;
					back_buffer [x + (back_lpitch * y)] = copy_buffer [x + (pitch * y)];
				}
				DDraw_SecondaryUnlock ();
				DDraw_Flip ();
				while ((GetTickCount() - timer)<(1000/30));
				timer = GetTickCount();
			}
			DDraw_UnlockSurface (copy);
			DDraw_Blit (0,0,800,600,copy,lpddsback,false);
			DDraw_Flip ();
			while ((GetTickCount() - timer)<500);
		} break;
	case TRANSITION_LIGHTENING:
		{
			for (int count = 0; count < (90+rand()%60); count++)
			{
				int x,y;
				x = 5 - rand()%10;
				y = 5 - rand()%10;
				DDraw_fillRect (0,0,800,600,0,lpddsback);
				DDraw_Blit (x,y,800,600,copy,lpddsback,false);
				if (!(rand()%15))
				{
					DDraw_fillRect (0,0,800,600,RGB216B(25,48,25),lpddsback);
				}
				DDraw_Flip ();
				while ((GetTickCount() - timer) < (1000/30));
				timer = GetTickCount();
			}
		} break;
	}
	copy->Release ();
	copy = NULL;
	return 1;
}

int DDraw_CopyBitmap(LPDIRECTDRAWSURFACE7 surface, HBITMAP hbm, int x, int y, int dx, int dy)
{
    HDC                     hdcImage;
    HDC                     hdc;
    BITMAP                  bm;
    DDSURFACEDESC2          ddsd;

    if (hbm == NULL || surface == NULL)
        return 0;
    //
    // Make sure this surface is restored.
    //
    surface->Restore ();
    //
    // Select bitmap into a memoryDC so we can use it.
    //
    hdcImage = CreateCompatibleDC(NULL);
    if (!hdcImage)
        return 0;
    SelectObject(hdcImage, hbm);
    //
    // Get size of the bitmap
    //
    GetObject(hbm, sizeof(bm), &bm);
    dx = dx == 0 ? bm.bmWidth : dx;     // Use the passed size, unless zero
    dy = dy == 0 ? bm.bmHeight : dy;
    //
    // Get size of surface.
    //
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
    surface->GetSurfaceDesc(&ddsd);

    if (surface->GetDC(&hdc) == DD_OK)
    {
        StretchBlt(hdc, 0, 0, ddsd.dwWidth, ddsd.dwHeight, hdcImage, x, y, dx, dy, SRCCOPY);
        surface->ReleaseDC(hdc);
    }
    DeleteDC(hdcImage);
    return 1;
}

int DDraw_LoadBitmap(LPDIRECTDRAWSURFACE7 surface, LPCSTR szBitmap)
{
    HBITMAP                 hbm;
    HRESULT                 hr;

    hbm = (HBITMAP) LoadImage(NULL, szBitmap, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION);
    if (hbm == NULL)
    {
        return 0;
    }
    hr = DDraw_CopyBitmap(surface, hbm, 0, 0, 0, 0);
    if (hr != DD_OK)
    {
		DeleteObject (hbm);
		return 0;
    }
    DeleteObject(hbm);
    return hr;
}

int DDraw_SaveGraphic (char *name, LPDIRECTDRAWSURFACE7 surface)
{
	ofstream out(name, ios::out | ios::binary);
	DDraw_InitStruct (ddsd);
	if (FAILED(surface->Lock (NULL,&ddsd,DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT,NULL)))
		return 0;
	out.write ((char *)ddsd.lpSurface, (ddsd.dwHeight * ddsd.dwWidth) * 2);
	surface->Unlock (NULL);
	out.close ();
	return 1;
}

int DDraw_LoadGraphic (char *name, int length, LPDIRECTDRAWSURFACE7 surface)
{
	ifstream in(name, ios::in | ios::binary);
	if (!in)
		return 0;

	DDraw_InitStruct (ddsd);
	if (FAILED(surface->Lock(NULL,&ddsd,DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT,NULL)))
		return 0;

	in.read((char *) ddsd.lpSurface, length);

	surface->Unlock (NULL);
	in.close ();
	return 1;
}

int DDraw_Blit (int x, int y, int w, int h, LPDIRECTDRAWSURFACE7 from, LPDIRECTDRAWSURFACE7 to, bool transparent)
{
	RECT dest;
	dest.top=y;
	dest.left=x;
	dest.bottom=y+h;
	dest.right=x+w;

	HRESULT hr;
	if (transparent)
		hr = to->Blt(&dest,from,NULL,(DDBLT_WAIT|DDBLT_KEYSRC),NULL);
	else
		hr = to->Blt(&dest,from,NULL,DDBLT_WAIT,NULL);

	if (hr!=DD_OK)
	{
		if (hr==DDERR_SURFACELOST)
		{
			return 2;	// Surface Lost
		}
		return 0;
	} // end if

	return 1;
}

int DDraw_Blit (bool style, int x, int y, int u, int v, int w, int h, LPDIRECTDRAWSURFACE7 from, LPDIRECTDRAWSURFACE7 to, bool transparent)
{
	// The more powerful Blit version

	// style true = scaling
	// style false = clipping

	RECT dest,src;
	if (style)
	{
		dest.top=y;
		dest.left=x;
		dest.bottom=y+h;
		dest.right=x+w;
		src.top=v;
		src.bottom=v+h;
		src.left=u;
		src.right=u+w;
	}
	else
	{
		dest.top=v;
		dest.left=u;
		dest.bottom=v+h;
		dest.right=u+w;
		src.top=y;
		src.bottom=y+h;
		src.left=x;
		src.right=x+w;
	}

	HRESULT hr;
	if (transparent)
		hr = to->Blt(&dest,from,&src,(DDBLT_WAIT|DDBLT_KEYSRC),NULL);
	else
		hr = to->Blt(&dest,from,&src,DDBLT_WAIT,NULL);

	if (hr != DD_OK)
	{
		if (hr == DDERR_SURFACELOST)
		{
			return 2;	// Surface Lost
		}
		return 0;
	} // end if
	return 1;
}

LPDIRECTDRAWSURFACE7 DDraw_CreateTransparentSurface (int w, int h)
{
	LPDIRECTDRAWSURFACE7 surface=NULL;

	DDraw_InitStruct (ddsd);
	ddsd.dwFlags=DDSD_CAPS|DDSD_WIDTH|DDSD_HEIGHT|DDSD_CKSRCBLT;
	ddsd.dwHeight=h;
	ddsd.dwWidth=w;
	ddsd.ddsCaps.dwCaps=DDSCAPS_OFFSCREENPLAIN;
	ddsd.ddckCKSrcBlt.dwColorSpaceLowValue=0;
	ddsd.ddckCKSrcBlt.dwColorSpaceHighValue=0;

	if (FAILED(lpdd->CreateSurface(&ddsd,&surface,NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to create offscreen surface",0);
		return NULL;
	} // end if
	return surface;
}

LPDIRECTDRAWSURFACE7 DDraw_CreateTransparentSurface (int w, int h, int flags)
{
	// special version
	LPDIRECTDRAWSURFACE7 surface=NULL;

	DDraw_InitStruct (ddsd);
	ddsd.dwFlags=DDSD_CAPS|DDSD_WIDTH|DDSD_HEIGHT|DDSD_CKSRCBLT;
	ddsd.dwHeight=h;
	ddsd.dwWidth=w;
	ddsd.ddsCaps.dwCaps=DDSCAPS_OFFSCREENPLAIN|flags;
	ddsd.ddckCKSrcBlt.dwColorSpaceLowValue=0;
	ddsd.ddckCKSrcBlt.dwColorSpaceHighValue=0;

	if (FAILED(lpdd->CreateSurface(&ddsd,&surface,NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to create offscreen surface",0);
		return NULL;
	} // end if
	return surface;
}

LPDIRECTDRAWSURFACE7 DDraw_CreateSurface (int w, int h, int flags)
{
	// special version
	LPDIRECTDRAWSURFACE7 surface=NULL;

	DDraw_InitStruct (ddsd);
	ddsd.dwFlags=DDSD_CAPS|DDSD_WIDTH|DDSD_HEIGHT;
	ddsd.dwHeight=h;
	ddsd.dwWidth=w;
	ddsd.ddsCaps.dwCaps=DDSCAPS_OFFSCREENPLAIN|flags;

	if (FAILED(lpdd->CreateSurface(&ddsd,&surface,NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to create offscreen surface",0);
		return NULL;
	} // end if
	return surface;
}

LPDIRECTDRAWSURFACE7 DDraw_CreateSurface (int w, int h)
{
	LPDIRECTDRAWSURFACE7 surface=NULL;

	DDraw_InitStruct (ddsd);
	ddsd.dwFlags=DDSD_CAPS|DDSD_WIDTH|DDSD_HEIGHT;
	ddsd.dwHeight=h;
	ddsd.dwWidth=w;
	ddsd.ddsCaps.dwCaps=DDSCAPS_OFFSCREENPLAIN;

	if (FAILED(lpdd->CreateSurface(&ddsd,&surface,NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to create offscreen surface",0);
		return NULL;
	} // end if
	return surface;
}

USHORT *DDraw_LockSurface (LPDIRECTDRAWSURFACE7 surface, int *lPitch)
{
	DDraw_InitStruct (ddsd);
	surface->Lock (NULL,&ddsd,DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT,NULL);

	*lPitch = (int) ddsd.lPitch / 2;
	return (USHORT *)ddsd.lpSurface;
}

int DDraw_UnlockSurface (LPDIRECTDRAWSURFACE7 surface)
{
	surface->Unlock (NULL);
	return 1;
}

int DDraw_AttachClipper (LPDIRECTDRAWSURFACE7 surface, int num_rects, LPRECT rect_list)
{
	int index;
	LPRGNDATA region_data;

	if (FAILED(lpdd->CreateClipper(0,&lpddclipper,NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to create clipper",0);
		return 0; 
	} // end if
	region_data = (LPRGNDATA)malloc(sizeof(RGNDATAHEADER)+num_rects*sizeof(RECT));
	memcpy(region_data->Buffer,rect_list,sizeof(RECT)*num_rects);

	region_data->rdh.dwSize=sizeof(RGNDATAHEADER);
	region_data->rdh.iType=RDH_RECTANGLES;
	region_data->rdh.nCount=num_rects;
	region_data->rdh.nRgnSize=num_rects*sizeof(RECT);
	region_data->rdh.rcBound.left=64000;
	region_data->rdh.rcBound.right=-64000;
	region_data->rdh.rcBound.top=64000;
	region_data->rdh.rcBound.bottom=-64000;

	for (index = 0; index<num_rects;index++)
	{
		if (rect_list[index].left < region_data->rdh.rcBound.left)
			region_data->rdh.rcBound.left = rect_list[index].left;
		if (rect_list[index].right > region_data->rdh.rcBound.right)
			region_data->rdh.rcBound.right = rect_list[index].right;
		if (rect_list[index].top < region_data->rdh.rcBound.top)
			region_data->rdh.rcBound.top = rect_list[index].top;
		if (rect_list[index].bottom > region_data->rdh.rcBound.bottom)
			region_data->rdh.rcBound.bottom = rect_list[index].bottom;
	}

	if (FAILED(lpddclipper->SetClipList(region_data,0)))
	{
		free(region_data);
		MessageBox (main_window_handle,"Error","Unable to set clip list",0);
		return 0;
	} // end if
	if (FAILED(surface->SetClipper(lpddclipper)))
	{
		free(region_data);
		MessageBox (main_window_handle,"Error","Unable to attach clipper",0);
		return 0;
	}

	free (region_data);
	return 1;
}

int DDraw_Flip ( void )
{
	while (FAILED(lpddsprimary->Flip(NULL,DDFLIP_WAIT)));
	return 1;
}

int DDraw_SecondaryLock ( void )
{
	DDraw_InitStruct(ddsd);
	if (FAILED(lpddsback->Lock(NULL,&ddsd,DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT,NULL)))
	{
		MessageBox(main_window_handle,"Error","Unable to lock secondary surface",0);
		return 0;
	} // end if
	back_buffer = (USHORT *)ddsd.lpSurface;
	back_lpitch = (ddsd.lPitch >> 1);
	return 1;
}

int DDraw_SecondaryUnlock ( void )
{
	if (FAILED(lpddsback->Unlock(NULL)))
	{
		MessageBox(main_window_handle,"Error","Unable to unlock lpddsback",0);
		return 0;
	}
	return 1;
}

int DDraw_PrimaryLock ( void )
{
	DDraw_InitStruct(ddsd);
	if (FAILED(lpddsprimary->Lock(NULL,&ddsd,DDLOCK_SURFACEMEMORYPTR|DDLOCK_WAIT,NULL)))
	{
		return 0;
	} // end if
	primary_lpitch = (ddsd.lPitch >> 1);
	primary_buffer = (USHORT *)ddsd.lpSurface;
	return 1;
}

int DDraw_PrimaryUnlock ( void )
{
	if (FAILED(lpddsprimary->Unlock(NULL)))
	{
		return 0;
	} // end if
	return 1;
}

int DDraw_Setup ( void )
{
	// First, get lpdd (revision 4 (DirectDraw 6.1))
	LPDIRECTDRAW lpddtemp = NULL;
	DDPIXELFORMAT ddpixel;	// used to check what 16b format

	if (FAILED(DirectDrawCreate(NULL,&lpddtemp,NULL)))
	{
		MessageBox(main_window_handle,"Error","Unable to create lpddtemp",0);
		return 0;
	} // end if
	if (FAILED(lpddtemp->QueryInterface(IID_IDirectDraw7,(LPVOID *)&lpdd)))
	{
		MessageBox(main_window_handle,"Error","Unable to obtain IDirectDraw7",0);
		return 0;
	} // end if
	lpddtemp->Release();

	// Now set up lpdd
	if (FAILED(lpdd->SetCooperativeLevel (main_window_handle,DDSCL_ALLOWMODEX|DDSCL_ALLOWREBOOT|DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN)))
	{
		MessageBox(main_window_handle,"Error","Unable to set Cooperative level",0);
		return 0;
	} // end if
	if (FAILED(lpdd->SetDisplayMode(xRes,yRes,bpp,0,0)))
	{
		MessageBox(main_window_handle,"Error","Unable to set display mode",0);
		return 0;
	} // end if

	// Time for the primary surface
	DDraw_InitStruct(ddsd);
	ddsd.dwFlags = DDSD_CAPS|DDSD_BACKBUFFERCOUNT;
	ddsd.dwBackBufferCount = 1;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE|DDSCAPS_COMPLEX|DDSCAPS_FLIP;
	if (FAILED(lpdd->CreateSurface(&ddsd,&lpddsprimary,NULL)))
	{
		MessageBox(main_window_handle,"Error","Unable to create lpddsprimary",0);
		return 0;
	} // end if

	// Now its time for the back buffer
	ddsd.ddsCaps.dwCaps=DDSCAPS_BACKBUFFER;
	if (FAILED(lpddsprimary->GetAttachedSurface(&ddsd.ddsCaps,&lpddsback)))
	{
		MessageBox(main_window_handle,"Error","Unable to get back buffer",0);
		return 0;
	} // end if

	// Now attach the clipper to the secondary surface
	RECT rect = {0,0,800,600};
	DDraw_AttachClipper (lpddsback,1,&rect);

	// Check to see if we are using 5.6.5 format
	DDraw_InitStruct (ddpixel);
	lpddsprimary->GetPixelFormat(&ddpixel);
	if (ddpixel.dwFlags & DDPF_RGB)
	{
		if (ddpixel.dwRGBBitCount != 16)
		{
			MessageBox (main_window_handle,"Error","5.5.5 16bit mode!",0);
			return 0;	// x.5.5.5
		}
	}

	// everything set and we are in 5.6.5 16b format
	return 1;
}

int DDraw_Release ( void )
{
	if (lpddclipper)
		lpddclipper->Release();
	if (lpddsback)
		lpddsback->Release();
	if (lpddsprimary)
		lpddsprimary->Release();
	if (lpdd)
		lpdd->Release();
	return 1;
}


// DirectInput functions
int DInput_KeyboardRead ( void )
{
	if (FAILED(lpdikey->GetDeviceState(sizeof(UCHAR)*256,(LPVOID)keystate)))
	{
		MessageBox (main_window_handle,"Error","Unable to read keyboard!",0);
		return 0;
	} // end if
	return 1;
}

int DInput_MouseRead ( void )
{
	if (FAILED(lpdimouse->GetDeviceState (sizeof(DIMOUSESTATE),&mousestate)))
	{
		MessageBox (main_window_handle,"Error","Unable to read mouse!",0);
		return 0;
	}
	return 1;
}

int DInput_Setup ( void )
{
	if (FAILED(DirectInputCreate(main_instance,DIRECTINPUT_VERSION,&lpdi,NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to create DirectInput Object",0);
		return 0;
	} // end if

	return 1;
}

int DInput_KeyboardSetup ( void )
{
	if (FAILED(lpdi->CreateDevice(GUID_SysKeyboard,&lpdikey,NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to create keyboard device",NULL);
		return 0;
	} // end if
	if (FAILED(lpdikey->SetCooperativeLevel(main_window_handle,DISCL_BACKGROUND|DISCL_NONEXCLUSIVE)))
	{
		MessageBox (main_window_handle,"Error","Unable to set cooperative level for lpdikey",0);
		return 0;
	} // end if
	if (FAILED(lpdikey->SetDataFormat(&c_dfDIKeyboard)))
	{
		MessageBox (main_window_handle,"Error","Unable to set data format for lpdikey",0);
		return 0;
	} // end if
	if (FAILED(lpdikey->Acquire()))
	{
		MessageBox (main_window_handle,"Error","Unable to acquire lpdikey!",0);
		return 0;
	} // end if
	return 1;
}

int DInput_MouseSetup ( void )
{
	if (FAILED(lpdi->CreateDevice(GUID_SysMouse,&lpdimouse,NULL)))
	{
		MessageBox(main_window_handle,"Error","Unable to get mouse device!",0);
		return 0;
	}
	if (FAILED(lpdimouse->SetCooperativeLevel (main_window_handle,DISCL_BACKGROUND|DISCL_NONEXCLUSIVE)))
	{
		MessageBox (main_window_handle,"Error","Unable to set mouse cooperative level!",0);
		return 0;
	}
	if (FAILED(lpdimouse->SetDataFormat(&c_dfDIMouse)))
	{
		MessageBox (main_window_handle,"Error","Unable to set mouse data format!",0);
		return 0;
	}
	if (FAILED(lpdimouse->Acquire()))
	{
		MessageBox (main_window_handle,"Error","unable to acquire mouse!",0);
		return 0;
	}
	return 1;
}

int DInput_Release ( void )
{
	if (lpdikey)
	{
		lpdikey->Unacquire ();
		lpdikey->Release ();
	}
	if (lpdimouse)
	{
		lpdimouse->Unacquire();
		lpdimouse->Release ();
	}
	if (lpdi)
		lpdi->Release ();
	return 1;
}