// Origionally by Andre' LaMothe
// Adapted for DXLib by Stefan Hajnoczi

// INCLUDES ///////////////////////////////////////////////
#define WIN32_LEAN_AND_MEAN  // just say no to MFC

#define INIT_GUID

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream> // include important C/C++ stuff
#include <time.h>
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>
#include <ddraw.h>
#include <dinput.h>
#include <new>
#include <winsock2.h>
#include "DXLib.h"
#include "resource.h"

// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "RPSS"
#define LEFT_BUTTON 0
#define RIGHT_BUTTON 1

#define WM_SOCKET	WM_USER+1	// Socket Windows Message

#define GS_TITLE	100			// Game States
#define GS_WAITFORPLAYER 101
#define GS_WAITFORINPUT 102

#define RPSS_NUMOFUSERS 0x03	// Protocol constants
#define RPSS_STARTGAME 0x04
#define RPSS_SCISSOR 0x05
#define RPSS_ROCK 0x06
#define RPSS_PAPER 0x07
#define RPSS_QUIT 0x08

// MACROS /////////////////////////////////////////////////
#define DDraw_InitStruct(ddstruct) { memset(&ddstruct,0,sizeof(ddstruct)); ddstruct.dwSize=sizeof(ddstruct); }
#define KEYDOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEYUP(vk_code)   ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)

// GLOBALS ////////////////////////////////////////////////
HWND      main_window_handle = NULL; // globally track main window
HINSTANCE main_instance      = NULL; // globally track hinstance

char txtbuffer[80];                     // general printing buffer
bool running = true;	// used to keep track of program state
bool paused = false;	// is set to true when Alt-Tab

bool receivedpacket = false; // used to remember if the opponent has finished yet
int p1 = 0, p2 = 0;	// scores

int choicep1, choicep2;	// what both players chose
int mouse_x = 0;	// mouse coordinates
int mouse_y = 0;

int GameState = GS_TITLE;	// Game state
DWORD fps, keydelay; // timers

int which_quad = 0;	// used for entering IP address of server and for keeping track of what action player will take
int ip[4];			// used to store IP address of server

LPDIRECTDRAWSURFACE7 rock = NULL;		// Rock icon
LPDIRECTDRAWSURFACE7 paper = NULL;		// Paper icon
LPDIRECTDRAWSURFACE7 scissor = NULL;	// Shoot icon
LPDIRECTDRAWSURFACE7 cursor = NULL;		// Cursor

// WinSock globals
SOCKET s;			// Socket handle
sockaddr_in you;	// Server's attributes

// FUNCTIONS //////////////////////////////////////////////

void ReleaseGraphics ( void )
{
	// Release Graphics here
	rock->Release ();
	paper->Release ();
	scissor->Release ();
	cursor->Release ();

	DDraw_Release ();
}

void SetupGraphics ( void )
{
	DDraw_Setup ();

	// Load Graphics here
	rock = DDraw_CreateSurface (128,128);
	DDraw_LoadBitmap (rock,"rock.bmp");

	paper = DDraw_CreateSurface (128,128);
	DDraw_LoadBitmap (paper,"paper.bmp");

	scissor = DDraw_CreateSurface (128,128);
	DDraw_LoadBitmap (scissor,"scissor.bmp");

	cursor = DDraw_CreateTransparentSurface (32,32);
	DDraw_LoadBitmap (cursor,"cursor.bmp");
}

LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT		ps;		// used in WM_PAINT
HDC				hdc;	// handle to a device context
char txtbuffer[80];        // used to print strings

// what is the message 
switch(msg)
	{	
	case WM_CREATE: 
        {
		// do initialization stuff here
        // return success
		return(0);
		} break;
   
	case WM_PAINT: 
		{
		// simply validate the window 
   	    hdc = BeginPaint(hwnd,&ps);	 
        
        // end painting
        EndPaint(hwnd,&ps);

        // return success
		return(0);
   		} break;

	case WM_DESTROY: 
		{

		// kill the application, this sends a WM_QUIT message 
		PostQuitMessage(0);
		running=false;
        // return success
		return(0);
		} break;

	case WM_KILLFOCUS:
		{
			if (!paused)
				paused = true;
			return 0;
		} break;

	case WM_SETFOCUS:
		{
			if (paused)
			{
				paused = false;
			}
			return 0;
		} break;
	case WM_SOCKET:
		{
			if (WSAGETSELECTERROR(lparam))
			{	// error!
				sprintf (txtbuffer,"Error2: %d",WSAGetLastError());
				MessageBox (main_window_handle,txtbuffer,"Error",MB_OK);
				PostQuitMessage (0);
				running = false;
				return 0;
			}
			switch (WSAGETSELECTEVENT(lparam))
			{
			case FD_READ:
				{
					char buffer[2];
					recv (s,buffer,2,0);

					switch (GameState)
					{
					case GS_TITLE:
						{
							if (buffer[0]==RPSS_NUMOFUSERS)
								GameState = GS_WAITFORPLAYER;
						} break;
					case GS_WAITFORPLAYER:
						{
							if (buffer[0]==RPSS_STARTGAME)
							{
								p1 = 0;	// reset scores
								p2 = 0;
								which_quad = 1;	// set to rock for next screen
								receivedpacket = false;
								GameState = GS_WAITFORINPUT;
							}
						} break;
					case GS_WAITFORINPUT:
						{
							switch (buffer[0])
							{
							case RPSS_ROCK:
								{
									choicep2 = 1;
								} break;
							case RPSS_PAPER:
								{
									choicep2 = 2;
								} break;
							case RPSS_SCISSOR:
								{
									choicep2 = 3;
								} break;
							default:
								{	// error
									PostQuitMessage (0);
									running = false;
									return 0;
								}
							}
							receivedpacket = true;
						}
					}
				} break;
			case FD_CLOSE:
				{	// opponent quit
					sprintf (txtbuffer,"%c%d",RPSS_QUIT,0);
					send (s,txtbuffer,2,0);	// tell server that we quit
					shutdown (s,SD_SEND);
					closesocket (s);
					PostQuitMessage (0);
					running = false;
					return 0;
				} break;
			}
		} break;
	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc

///////////////////////////////////////////////////////////

int Game_Main(void *parms = NULL, int num_parms = 0)
{
	if (DDraw_fillRect (0,0,800,600,0,lpddsback)==2)
	{
		// there was an Alt-Tab
		ReleaseGraphics ();
		SetupGraphics ();
	}

	switch (GameState)
	{
	case GS_TITLE:
		{	// Draw title screen and handle input
			DDraw_drawString ("Rock, Paper, Scissor, Shoot!",300,50,RGB(130,0,0),lpddsback);
			DDraw_drawString ("IP address of server:      .      .      .",300,100,RGB(130,0,0),lpddsback);
			for (int a=0; a<4; a++)
			{
				COLORREF c = RGB(130,0,0);
				if (which_quad==a)
				{
					c = RGB(0,130,0);
				}
				sprintf (txtbuffer,"%d",(int)ip[a]);
				DDraw_drawString (txtbuffer,435+(a*28),100,c,lpddsback);
			}
			DDraw_fillRect (325,130,475,145,RGB216B(0,40,5),lpddsback);
			DDraw_drawString ("Connect to address...",330,130,RGB(130,0,0),lpddsback);

			// Test for input
			DInput_MouseRead ();		// handle mouse related stuff
			mouse_x += mousestate.lX;
			mouse_y += mousestate.lY;

			if (mouse_y < 0)			// bounds check the cursor
				mouse_y = 0;
			if (mouse_x < 0)
				mouse_x = 0;
			if (mouse_x > 768)
				mouse_x = 768;
			if (mouse_y > 568)
				mouse_y = 568;

			if (mousestate.rgbButtons[LEFT_BUTTON] & 0x80)
			{	// click
				if ((mouse_x > 325)&&(mouse_x < 475)&&(mouse_y > 130)&&(mouse_y < 145))
				{	// "Connect to address..." was clicked
					// put together IP address of server in a string
					char a[16];
					sprintf (a,"%d.%d.%d.%d",(int)ip[0],(int)ip[1],(int)ip[2],(int)ip[3]);

					DDraw_drawString (a,100,100,RGB(0,255,255),lpddsback);
					// connect to server
					you.sin_family = AF_INET;
					you.sin_port = htons (5555);	// RPSS port is 5555
					you.sin_addr.s_addr = inet_addr (a);
	
					if (connect(s,(LPSOCKADDR)&you,sizeof(you))==SOCKET_ERROR)
					{
						if (WSAGetLastError()==WSAEWOULDBLOCK)
						{
							Sleep (750);
							connect(s,(LPSOCKADDR)&you,sizeof(you));
							break;
						}
						sprintf (txtbuffer,"Error: %d",WSAGetLastError());
						MessageBox (main_window_handle,txtbuffer,"Error",MB_OK);
						PostQuitMessage (0);
						running=false;
						return 0;
					}

					// successful connection
				}
			}
			DDraw_Blit (mouse_x,mouse_y,32,32,cursor,lpddsback,true);

			if ((GetTickCount() - keydelay) > 75)
			{
				DInput_KeyboardRead ();
				if (keystate[DIK_ESCAPE] & 0x80)
				{
					PostQuitMessage (0);
					running = false;
					return 0;
				}
				if (keystate[DIK_LEFT] & 0x80)
				{
					if (which_quad > 0)
						which_quad--;
				}
				if (keystate[DIK_RIGHT] & 0x80)
				{
					if (which_quad < 3)
						which_quad++;
				}
				if (keystate[DIK_UP] & 0x80)
				{
					if ((int)ip[which_quad] < 255)
						ip[which_quad]++;
				}
				if (keystate[DIK_DOWN] & 0x80)
				{
					if ((int)ip[which_quad] > 0)
						ip[which_quad]--;
				}
				keydelay = GetTickCount();
			}
		} break;
	case GS_WAITFORPLAYER:
		{
			DDraw_drawString ("Waiting for another player to join...",275,375,RGB(130,0,0),lpddsback);
			DDraw_drawString ("(Press ESCAPE to quit)",320,400,RGB(130,0,0),lpddsback);

			DInput_KeyboardRead ();
			if (keystate[DIK_ESCAPE] & 0x80)
			{
				PostQuitMessage (0);
				running = false;
				return 0;
			}
		} break;
	case GS_WAITFORINPUT:
		{
			sprintf (txtbuffer,"You: %d   Opponent: %d",p1,p2);
			DDraw_drawString (txtbuffer,250,12,RGB(130,0,0),lpddsback);

			if (choicep1<1)
			{
				int w = 128;
				if (which_quad == 1)
					w = 150;
				DDraw_Blit (60,216,w,w,rock,lpddsback,false);
				w = 128;
				if (which_quad == 2)
					w = 150;
				DDraw_Blit (208,216,w,w,paper,lpddsback,false);
				w = 128;
				if (which_quad == 3)
					w = 150;
				DDraw_Blit (358,216,w,w,scissor,lpddsback,false);

				if ((GetTickCount() - keydelay) > 75)
				{
					DInput_KeyboardRead ();
					if (keystate[DIK_ESCAPE] & 0x80)
					{
						char buffer[1];
						buffer[0] = RPSS_QUIT;
						send (s,buffer,1,0);
						shutdown (s,0);
						closesocket (s);
						PostQuitMessage (0);
						running = false;
						return 1;
					}
					if (keystate[DIK_LEFT] & 0x80)
					{
						if (which_quad>1)
							which_quad--;
					}
					if (keystate[DIK_RIGHT] & 0x80)
					{
						if (which_quad < 3)
							which_quad++;
					}
					if (keystate[DIK_RETURN] & 0x80)
					{
						choicep1 = which_quad;
						switch (which_quad)
						{
						case 1:
							{
								sprintf (txtbuffer,"%c%d",RPSS_ROCK,0);
							} break;
						case 2:
							{
								sprintf (txtbuffer,"%c%d",RPSS_PAPER,0);
							} break;
						case 3:
							{
								sprintf (txtbuffer,"%c%d",RPSS_SCISSOR,0);
							} break;

						}
						send (s,txtbuffer,2,0);
					}
					keydelay = GetTickCount();
				}
			}
			else
			{
				switch (choicep1)
				{
				case 1:
					{
						DDraw_Blit (336,216,128,128,rock,lpddsback,false);
					} break;
				case 2:
					{
						DDraw_Blit (336,216,128,128,paper,lpddsback,false);
					} break;
				case 3:
					{
						DDraw_Blit (336,216,128,128,scissor,lpddsback,false);
					} break;
				}
			}

			if (receivedpacket)
			{
			if (choicep1 > 0)	// both players are done
			{
				receivedpacket = false;
				switch (choicep1)
				{
				case 1:
					{
						if (choicep2==3)
							p1++;
						if (choicep2==2)
							p2++;
					} break;
				case 2:
					{
						if (choicep2==1)
							p1++;
						if (choicep2==3)
							p2++;
					} break;
				case 3:
					{
						if (choicep2==2)
							p1++;
						if (choicep2==1)
							p2++;
					} break;
				}
				choicep1 = 0;
				choicep2 = 0;

				// next turn...
			}
			}
		}
	}

	DDraw_Flip ();
	while ((GetTickCount() - fps) < (1000/30));
	fps = GetTickCount();
	return(1);

} // end Game_Main

////////////////////////////////////////////////////////////

int Game_Init(void *parms = NULL, int num_parms = 0)
{
	srand (time(0));

	// Initialize DirectX here
	ShowCursor (false);
	TableSetup ();
	SetupGraphics ();	// Setup DirectDraw and Graphics
	DInput_Setup ();
	DInput_MouseSetup ();
	DInput_KeyboardSetup ();

	// Setup WinSock
	WSADATA w;
	int error = WSAStartup (0x0202,&w);
	if (error)
	{
		MessageBox (main_window_handle,"Error: You need WinSock 2.2!","Error",MB_OK);
		PostQuitMessage (0);
		running = false;
		return 0;
	}
	if (w.wVersion!=0x0202)
	{
		MessageBox (main_window_handle,"Error:  Wrong WinSock version!","Error",MB_OK);
		PostQuitMessage (0);
		running = false;
		WSACleanup ();
		return 0;
	}

	s = socket (AF_INET,SOCK_STREAM,0); // create socket

	// Make socket asynchronous and hook to WM_SOCKET message defined earlier
	WSAAsyncSelect (s,main_window_handle,WM_SOCKET,(FD_CLOSE | FD_CONNECT | FD_READ));

	// set default IP address to 127.0.0.1 (localhost)
	ip[0] = (char)127;
	ip[3] = (char)1;
	fps = GetTickCount ();
	keydelay = fps;

	return(1);

} // end Game_Init

/////////////////////////////////////////////////////////////

int Game_Shutdown(void *parms = NULL, int num_parms = 0)
{
	WSACleanup ();	// Release WinSock

	// Release DirectX and show mouse
	ShowCursor (true);
	ReleaseGraphics ();	// Release DirectDraw and Graphics
	DInput_Release ();

	return(1);

} // end Game_Shutdown

// WINMAIN ////////////////////////////////////////////////
int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{

WNDCLASSEX winclass; // this will hold the class we create
HWND	   hwnd;	 // generic window handle
MSG		   msg;		 // generic message
HDC        hdc;      // graphics device context

// first fill in the window class stucture
winclass.cbSize         = sizeof(WNDCLASSEX);
winclass.style			= CS_DBLCLKS | CS_OWNDC | 
                          CS_HREDRAW | CS_VREDRAW;
winclass.lpfnWndProc	= WindowProc;
winclass.cbClsExtra		= 0;
winclass.cbWndExtra		= 0;
winclass.hInstance		= hinstance;
winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
winclass.hCursor		= LoadCursor(NULL, IDC_ARROW); 
winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
winclass.lpszMenuName	= NULL;
winclass.lpszClassName	= WINDOW_CLASS_NAME;
winclass.hIconSm        = LoadIcon(NULL, IDI_APPLICATION);

// save hinstance in global
main_instance = hinstance;

// register the window class
if (!RegisterClassEx(&winclass))
	return(0);

// create the window
if (!(hwnd = CreateWindowEx(NULL,                  // extended style
                            WINDOW_CLASS_NAME,     // class
						    "RPSS", // title
						    WS_POPUPWINDOW | WS_VISIBLE,
					 	    0,0,	  // initial x,y
						    800,600,  // initial width, height
						    NULL,	  // handle to parent 
						    NULL,	  // handle to menu
						    hinstance,// instance of this application
						    NULL)))	// extra creation parms
return(0);

// save main window handle
main_window_handle = hwnd;

// initialize game here
Game_Init();

// enter main event loop
while(TRUE)
	{
    // test if there is a message in queue, if so get it
	if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
	   { 
	   // test if this is a quit
       if (msg.message == WM_QUIT)
           break;
	
	   // translate any accelerator keys
	   TranslateMessage(&msg);

	   // send the message to the window proc
	   DispatchMessage(&msg);
	   } // end if
    
	if ((running) && (!paused))
	{
		Game_Main ();
	}
	else
	{
		WaitMessage ();
	}
       
	} // end while

// closedown game here

Game_Shutdown();

// return to Windows like this
return(msg.wParam);

} // end WinMain

///////////////////////////////////////////////////////////